/* NAME:
 *	unfsc - User NFS Client
 *
 * SYNOPSIS:
 *	unfsc [options] "server:filesystem" "mountpt" [unfs "mntopts"]
 *
 * DESCRIPTION:
 *	This is the client side of my secure NFS.
 *
 *	Mounting a filesystem can be achieved by running 'unfsc'
 *	directly from the command line, or via 'mount' by linking
 *	'unfsc' to the name 'mount_unfs' in the appropriate directory.
 *	This currently works for 4.4BSD and SunOS.  Other systems may
 *	need to use a shell script to massage arguments provided by
 *	'mount'. 
 *
 *	Unmounting can be done by killing 'unfsc' with a SIGTERM as
 *	happens when the system shuts down.  On receipt of a SIGTERM
 *	'unfsc' will send a UMNT RPC to the server, and try very hard
 *	to unmount its filesystem from the kernel before exiting.
 *	On SunOS and other system where 'umount' looks for 'umount_*'
 *	in the same way 'mount' does, you can link 'unfsc' to
 *	'umount_unfs' in the appropriate directory and simply use
 *	'umount'. 
 *
 *	'Unfsc' supports a reasonable subset of the options normally
 *	supported by NFS.  The options are listed below.
 *	
 *	If using UDP transport or TCP on a kernel that supports TCP
 *	and the 'auth' feature is not being used, 'unfsc' behaves
 *	exactly like 'mount_nfs'.  It obtains a file handle from the
 *	server via a MNT RPC and passes the handle and sockaddr to the
 *	kernel. 
 *
 *	If using 'auth' or TCP transport on a system that does not
 *	support it directly, we register a UDP based NFS service with
 *	the kernel and fork a child to to shuffle RPC's to the server.
 *
 *	Options:
 *
 *	-d	Enable some debuging information.
 *		
 *	-u	Unmount "mountpt".  Being invoked with a name that
 *		starts with 'umount' has the same effect.
 *
 *	-P "port"
 *		Or 'port='"port" in "mntopts".  Server is listening on
 *		"port".  Recall that we do not use theportmapper at
 *		all.
 *
 *	-a	Or 'auth' present in "mntopts". Server requires
 *		authentication, before it will issue a filesystem
 *		handle.  Enter an authorization loop before issuing
 *		the MNT RPC.  Only used with TCP.
 *
 *	-b	Or 'bg' in "mntopts". Carry on in background if server
 *		does not respond.
 *
 *	-i	Make operations on filesystem interruptible. 
 *
 *	-T	Or 'tcp' in "mntopts". Use TCP as transport.
 *
 *	-s	Or 'soft' in "mntopts". Soft mount point.
 *
 *	-o "mntopts"
 *		Pass "mntopts" to kernel (in general anyway).
 *
 *	-t "timeo"
 *		Or 'timeo='"timeo" in "mntopts".
 *
 *	-r "rsize"
 *		Or 'rsize='"rsize" in "mntopts".
 *
 *	-w "wsize"
 *		Or 'wsize='"wsize" in "mntopts". 
 *
 *	Normally if we lose our connection to the server, we
 *	re-connect automagically.  If using 'auth' re-connecting will
 *	fail, so when we lose the server, we try to unmount the
 *	filesystem and exit.
 *
 * SEE ALSO:
 *	unfsd(8)
 *
 *	
 * AUTHOR:
 *	Simon J. Gerraty <sjg@zen.void.oz.au>
 */
/*
 *      @(#)Copyright (c) 1994, Simon J. Gerraty.
 *      
 *      This code is presented AS IS.  It comes with NO WARRANTY.
 *      Permission to use, modify and distribute this source code 
 *      is granted subject to the following conditions.
 *      1/ that the above copyright notice and this notice 
 *      are preserved in all copies and that due credit be given 
 *      to the author.  
 *      2/ that any changes to this code are clearly commented 
 *      as such so that the author does not get blamed for bugs 
 *      other than his own.
 *      
 *      Please send copies of changes and bug-fixes to:
 *      sjg@zen.void.oz.au
 */

#ifndef lint
static char RCSid[] = "$Id: unfsc.c,v 1.6 1995/05/13 07:23:03 sjg Exp $";
#endif
#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <sys/syslog.h>
#include <sys/types.h>
#include <sys/mount.h>
#include <netdb.h>
#include <errno.h>

#include "mntopts.h"
#include "unfsd.h"

/*
 * some mount options to handle assignments
 */
#define M_BG	 0x800000
#define M_TCP 	 0x400000
#define M_AUTH	 0x200000
#define M_INTR	 0x100000

/*
 * just in case we are linked to mount_unfs under SunOS
 */
#define MOPT_TCP		{ "tcp",	0, M_TCP }
#define MOPT_AUTH		{ "auth",	0, M_AUTH }
#define MOPT_BG			{ "bg",		0, M_BG }
#define MOPT_INTR		{ "intr",	0, M_INTR }

#define MOPT_PORT		{ "port",	OPT_ARG, &UnfsPort }
#define MOPT_RSIZE		{ "rsize",	OPT_ARG, &UnfsRsize }
#define MOPT_WSIZE		{ "wsize",	OPT_ARG, &UnfsWsize }
#define MOPT_TIMEO		{ "timeo",	OPT_ARG, &UnfsTimeo }



struct mntopt mopts[] = {
	MOPT_STDOPTS,
	MOPT_BG,
	MOPT_INTR,
	MOPT_TCP,
	MOPT_AUTH,
	MOPT_PORT,
	MOPT_RSIZE,
	MOPT_WSIZE,
	MOPT_TIMEO,
#ifdef MOPT_FORCE
	MOPT_FORCE,
	MOPT_UPDATE,
#endif
	{ (char *) 0, 0, 0 }
};

int
main(argc, argv)
	int argc;
	char **argv;
{
#ifndef BSD4_4
	extern	char *__progname;		/* warn*() want this */
#endif
	extern int optind, getopt();
	extern char *optarg;
	struct servent *sp = 0;
	char *options = "adbiTuP:r:t:w:so:";
	int c;
	int mntflags = 0;
	
	UnfsFlags = 0;
	UnfsMntSpec = UnfsMntPt = NULL;

	if (Myname = strrchr(argv[0], '/'))
		Myname++;
	else
		Myname = argv[0];
#ifndef BSD4_4
	__progname = Myname;
#endif
	
	UnfsPort = UnfsTimeo = UnfsRsize = UnfsWsize = -1;
	
	while ((c = getopt(argc, argv, options)) != EOF) {
		switch (c) {
		case 'd':
			UnfsFlags |= UNFS_DEBUG;
			break;
		case 'a':
#ifndef NO_FWTK
			UnfsFlags |= UNFS_AUTH;
#endif
			break;
		case 'b':
			UnfsFlags |= UNFS_BGRND;
			break;
		case 'i':
			UnfsFlags |= UNFS_INTR;
			break;
		case 's':
			UnfsFlags |= UNFS_SOFT;
			break;
		case 'T':
			UnfsFlags |= UNFS_TCP;
			break;
		case 'u':
			UnfsFlags |= UNFS_UNMOUNT;
			break;
		case 't':
			UnfsTimeo = atoi(optarg);
			break;
		case 'r':
			UnfsRsize = atoi(optarg);
			break;
		case 'w':
			UnfsWsize = atoi(optarg);
			break;
		case 'o':
			getmntopts(optarg, mopts, &mntflags);
			break;
		case 'P':
			UnfsPort = atoi(optarg);
			break;
		}
	}
	UnfsMntSpec = argv[optind];
	UnfsMntPt = argv[optind + 1];

	if (argc - optind > 3) {
		/*
		 * handle SunOS style mount opts
		 */
		getmntopts(argv[optind + 3], mopts, &mntflags);
	}
	
	if (mntflags & M_TCP)
		UnfsFlags |= UNFS_TCP;
	if (mntflags & M_BG)
		UnfsFlags |= UNFS_BGRND;
	if (mntflags & M_INTR)
		UnfsFlags |= UNFS_INTR;
#ifndef NO_FWTK
	if (mntflags & M_AUTH)
		UnfsFlags |= UNFS_AUTH;
#endif
	mntflags &= ~(M_TCP|M_BG|M_AUTH|M_INTR);

	if (strncmp(Myname, "umount", 6) == 0)
		UnfsFlags |= UNFS_UNMOUNT;
	
	if (UnfsFlags & UNFS_TCP) {
#ifdef KERNEL_TCP_NFS
		if (UnfsFlags & UNFS_AUTH)
#endif
			UnfsFlags |= UNFS_CLNT;
	}
	if (UnfsPort < 0) {
		if (UnfsFlags & UNFS_TCP)
			sp = getservbyname("unfs", "tcp");
		else
			sp = getservbyname("unfs", "udp");
		if (sp)
			UnfsPort = ntohs(sp->s_port);
#ifdef UNFS_DEFAULT_PORT
		else
			UnfsPort = UNFS_DEFAULT_PORT;
#endif
	}
	if (UnfsPort < 0)
		err(1, "%s: port unknown\n", Myname);
		
	if (UnfsFlags & UNFS_UNMOUNT)
		c = umount_unfs();
	else 
		c = mount_unfs(mntflags);
	exit(c);
}
